import { ConfigProvider, Tabs } from "antd";
import React, { useEffect, useMemo, useState } from "react";
import PageStyle from "./PageStyle";
import GraphStyle from "./GraphStyle";
import NodeAttrs from "@/components/NodeAttrs";
import { useModel } from "umi";
import InsetCss from "insert-css";
import Chat from "@/components/ai/Chat";
import { debounce, set } from "lodash-es";

InsetCss(`
  .shalu-tabs {
    display: flex;
    flex-direction: column;
    height: 100%;
  }
  .shalu-tabs-content-holder {
    flex: 1;
    overflow-y: auto;
  }
  `);
export default function Config() {
  const { selectedCell } = useModel("graphModel");
  const {
    rightPanelTabActiveKey,
    setRightPanelTabActiveKey,
    activeAIChat,
    setActiveAIChat,
    rightPanelWidth,
    setRightPanelWidth,
  } = useModel("appModel");

  const firstNode = useMemo(() => {
    return selectedCell.find((cell) => cell.isNode());
  }, [selectedCell]);

  const resizeRef = React.useRef<HTMLDivElement>(null);

  // 设置节点属性
  const handleSetNodeAttr = (attrs: any) => {
    firstNode?.setData({
      attrs,
    });
  };

  const tabItems = [
    {
      key: "1",
      label: `页面样式`,
      children: (
        <ConfigProvider
          componentSize="small"
          theme={{
            token: {
              colorPrimary: "#1890ff",
            },
          }}
        >
          <PageStyle />
        </ConfigProvider>
      ),
    },
    {
      key: "2",
      label: `图形样式`,
      children: (
        <ConfigProvider
          componentSize="small"
          theme={{
            token: {
              colorPrimary: "#1890ff",
            },
          }}
        >
          <GraphStyle />
        </ConfigProvider>
      ),
    },
    {
      key: "3",
      label: `节点属性`,
      children: (
        <ConfigProvider
          componentSize="small"
          theme={{
            token: {
              colorPrimary: "#1890ff",
            },
          }}
        >
          <NodeAttrs cell={firstNode} onChange={handleSetNodeAttr} />
        </ConfigProvider>
      ),
    },
  ];

  // useEffect(() => {
  //   if (selectedCell?.length) {
  //     setRightPanelTabActiveKey("2");
  //   } else {
  //     setRightPanelTabActiveKey("1");
  //   }
  // }, [selectedCell]);

  const [dragging, setDragging] = useState(false);

  // 拖拽改变右侧面板宽度
  const handleMouseMove = (e: MouseEvent) => {
    const offset = startRef.current - e.clientX;
    requestAnimationFrame(() => {
      setRightPanelWidth((w) => {
        const newW = w + offset;
        if (newW < 280) return 280;
        if (newW > window.innerWidth * 0.5) return window.innerWidth * 0.5;
        return newW;
      });
      startRef.current = e.clientX;
    });
  };

  const handleMouseUp = () => {
    setDragging(false);
    document.body.style.cursor = "default";
    document.removeEventListener("mousemove", handleMouseMove);
    document.removeEventListener("mouseup", handleMouseUp);
  };

  const startRef = React.useRef<number>(0);
  // 开启左右拖拽
  const handleMouseDown = (e: React.MouseEvent) => {
    setDragging(true);
    // 设置鼠标样式
    document.body.style.cursor = "ew-resize";
    startRef.current = e.clientX;
    document.addEventListener("mousemove", handleMouseMove);
    document.addEventListener("mouseup", handleMouseUp);
  };

  return (
    <div className="w-full h-full flex">
      <div
        className="h-full w-4px cursor-e-resize hover:bg-blue flex-shrink-0"
        style={{ backgroundColor: dragging ? "#60a5fa" : "" }}
        ref={resizeRef}
        onMouseDown={handleMouseDown}
      ></div>
      <div className="flex-1 overflow-hidden">
        {activeAIChat ? (
          <Chat onClose={() => setActiveAIChat(false)} />
        ) : (
          <Tabs
            centered
            items={tabItems}
            activeKey={rightPanelTabActiveKey}
            onChange={(key) => setRightPanelTabActiveKey(key)}
            className="w-full"
          />
        )}
      </div>
    </div>
  );
}
